//  -*- c++ -*-

#ifndef XPATH_VALUE_H
#define XPATH_VALUE_H

#include <qstring.h>
#include <qcstring.h>

#include <dom/dom_node.h>
#include <xpath/xpath_node.h>

namespace XPath {

class Exception;
class NodeSet;
class Boolean;
class Number;
class String;

/**
 * An XPath value.
 * See REC-xpath-19991116 1.0 Section 1.
 *
 * @version $Id: xmldemo.h,v 1.3 2002/02/18 00:45:47 rich Exp $
 * @author Richard Moore, rich@kde.org 
 */
class Value : public XPathNode
{
public:
    enum ValueType {
	ValueNull, ValueException, ValueNodeSet, ValueBoolean, ValueNumber, ValueString
    };

    Value() : tp(ValueNull) {}
    Value( const Value &res );
    virtual ~Value() {}

    ValueType type() const { return tp; }

    Exception toException();
    NodeSet toNodeSet();
    Boolean toBoolean();
    Number toNumber();
    String toString();

    virtual QString qstring() const;

protected:
    Value( ValueType t ) : tp(t) {}

private:
    ValueType tp;
};


/**
 * An XPath String value.
 * See REC-xpath-19991116 1.0 Section 1.
 */
class String : public Value
{
public:
    String() : Value(ValueString) {}
    String( const String &s ) : Value(ValueString), val(s.val) {}
    String( const QString &s ) : Value(ValueString), val(s) {}
    String( const char *s ) : Value(ValueString), val(s) {}
    virtual ~String() {}

    virtual QString qstring() const;

private:
    QString val;
};

/**
 * An error value returned by the XPath engine.
 */
class Exception : public Value
{
public:
    Exception() : Value(ValueException) {}
    Exception( const Exception &x ) : Value(ValueException), msg(x.msg) {}
    Exception( const QString &m ) : Value(ValueException), msg(m) {}
    virtual ~Exception() {}

    virtual QString qstring() const { return QString("Error: %1").arg(msg); }
    QString message() const { return msg; }

private:
    QString msg;
};

/**
 * An XPath node-set value.
 * See REC-xpath-19991116 1.0 Section 1.
 */
class NodeSet : public Value
{
public:
    NodeSet();
    NodeSet( const NodeSet &ns );
    virtual ~NodeSet();

private:
    DOM::NodeList val;
};

/**
 * An XPath Boolean value.
 * See REC-xpath-19991116 1.0 Section 1.
 */
class Boolean : public Value
{
public:
    Boolean();
    Boolean( const Boolean &b );
    virtual ~Boolean();

    virtual QString qstring() const { return QString("Boolean: %1").arg(val); }

private:
    bool val;
};

/**
 * An XPath Number value.
 * See REC-xpath-19991116 1.0 Section 1.
 */
class Number : public Value
{
public:
    Number() : Value(ValueNumber) {}
    Number( const Number &n ) : Value(ValueNumber) { this->val = n.val; }

    virtual QString qstring() const { return QString("Number: %1").arg(val); }

private:
    double val;
};

}; // namespace XPath

#endif // XPATH_VALUE_H

// Local Variables:
// c-basic-offset: 4
// End:
