/*
 *  This file is part of the KDE libraries
 *  Copyright (C) 2004 Richard Moore, rich@kde.org
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <math.h>

#include "xpath_value.h"
#include "xpath_function.h"

namespace XPath {

FunctionCall::FunctionCall( FunctionCallId func )
    : ExprNode(),
      id( func )
{
}

FunctionCall::~FunctionCall()
{
}

Value FunctionCall::evaluate() const
{
    switch( id ) {
	case FunctionBoolean:
	{
	    if ( !argCount() )
		return Boolean();

	    return argBoolean( 0 );
	}
	break;
	case FunctionCeiling:
	{
	    if ( !argCount() )
		return Number();

	    double val = argNumber( 0 ).value();
	    return Number( ceil(val) );
	}
	break;
	case FunctionConcat:
	{
	    DOM::DOMString res;

	    for ( int i = 0; i < argCount(); i++ ) {
		String s = argString(i);
		res += s.value();
	    }

	    return String( res );
	}
	break;
	case FunctionContains:
	{
	    if ( argCount() == 2 ) {
		QString s = argString(0).value().string();
		QString t = argString(1).value().string();

		return Boolean( s.contains( t ) );
	    }

	    return Boolean( false );
	}
	break;
	case FunctionCount:
	    break;
	case FunctionFalse:
	    return Boolean( false );
	    break;
	case FunctionFloor:
	{
	    if ( !argCount() )
		return Number();

	    double val = argNumber( 0 ).value();
	    return Number( floor(val) );
	}
	break;
	case FunctionId:
	    break;
	case FunctionLang:
	    break;
	case FunctionLast:
	    break;
	case FunctionLocalName:
	    break;
	case FunctionName:
	    break;
	case FunctionNamespaceURI:
	    break;
	case FunctionNormalizeSpace:
	{
	    if ( !argCount() )
		return String();
	    QString s = argString(0).value().string();
	    s = s.simplifyWhiteSpace();
	    return String( DOM::DOMString(s) );
	}
	break;
	case FunctionNot:
	{
	    if ( !argCount() )
		return Boolean();

	    Boolean b = argBoolean( 0 );
	    return Boolean( !b.value() );
	}
	break;
	case FunctionNumber:
	{
	    if ( !argCount() )
		return Number();

	    return argNumber( 0 );
	}
	break;
	case FunctionPosition:
	    break;
	case FunctionRound:
	{
	    if ( !argCount() )
		return Number();

	    double val = argNumber( 0 ).value();
	    return Number( round(val) );
	}
	break;
	case FunctionStartsWith:
	{
	    if ( argCount() == 2 ) {
		QString s = argString(0).value().string();
		QString t = argString(1).value().string();

		return Boolean( s.startsWith( t ) );
	    }

	    return Boolean( false );
	}
	break;
	case FunctionString:
	{
	    if ( !argCount() )
		return String();

	    return argString( 0 );
	}
	break;
	case FunctionStringLength:
	{
	    if ( !argCount() )
		return Number();

	    DOM::DOMString s = argString(0).value();
	    return Number( s.length() );
	}
	break;
	case FunctionSubstring:
	{
	    QString s = argString(0).value().string();
	    int start = argNumber(1).intValue();
	    if ( argCount() == 2 )
		return String( s.mid( start ) );
	    else {
		Number n( argNumber(2) );
		return String( s.mid( start, n.intValue() ) );
	    }
	    return String();
	}
	break;
	case FunctionSubstringAfter: 
	    if ( argCount() == 2 ) {
		QString s = argString(0).value().string();
		QString t = argString(1).value().string();

		int idx = s.find( t );
		if ( idx >= 0 ) {
		    idx = idx + t.length();
		    return String( s.mid(idx) );
		}
	    }
	    return String();
	    break;
	case FunctionSubstringBefore:
	    if ( argCount() == 2 ) {
		QString s = argString(0).value().string();
		QString t = argString(1).value().string();

		int idx = s.find( t );
		if ( idx >= 0 )
		    return String( s.left(idx) );
	    }
	    return String();
	    break;
	case FunctionSum: 
	    break;
	case FunctionTranslate: 
	    break;
	case FunctionTrue:
	    return Boolean( true );
	    break;
	default:
	    break;
    }
}

} // namespace XPath

// Local Variables:
// c-basic-offset: 4
// End:
