/*
 *  This file is part of the KDE libraries
 *  Copyright (C) 2004 Richard Moore, rich@kde.org
 *  Copyright (C) 2004 Zack Rusin, zack@kde.org
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <float.h>
#include <math.h>
#include <qstring.h>

#include "xpath_value.h"

namespace XPath {

Value::Value( const Value &res )
{
    impl = res.impl;
    if ( impl ) impl->ref();
}

Value::Value( ValueImpl * i )
{
    impl = i;
    impl->ref();
}

Value::~Value()
{
    if ( impl ) impl->deref();
}

Value &Value::operator=( const Value &other )
{
    if ( impl != other.impl ) {
        if(impl) impl->deref();
        impl = other.impl;
        if(impl) impl->ref();
    }
    return *this;
}

String Value::toString() const
{
    return String( impl->toString() );
}

Boolean Value::toBoolean() const
{
    return Boolean( impl->toBoolean() );
}

Number Value::toNumber() const
{
    return Number( impl->toNumber() );
}

//
// XPath NodeSet type
//

NodeSet::NodeSet()
    : Value( new NodeSetImpl() )
{
}

NodeSet::NodeSet( ValueImpl *impl )
    : Value( impl )
{
}

NodeSet::~NodeSet()
{
}

NodeSet &NodeSet::operator=( const NodeSet &other )
{
    if ( impl != other.impl ) {
        if(impl) impl->deref();
        impl = other.impl;
        if(impl) impl->ref();
    }
    return *this;
}

DOM::NodeList NodeSet::value() const
{
    return static_cast<NodeSetImpl*>( impl )->value();
}

//
// XPath Boolean type
//

Boolean::Boolean()
    : Value( new BooleanImpl() )
{
}

Boolean::Boolean( ValueImpl *impl )
    : Value( impl )
{
}

Boolean::Boolean( bool value )
    : Value( new BooleanImpl( value ) )
{
}

Boolean::Boolean( const Boolean &src )
    : Value( src )
{
}

Boolean::~Boolean()
{
}

Boolean &Boolean::operator=( const Boolean &other )
{
    Value::operator =(other);
    return *this;
}

bool Boolean::value() const
{
    return static_cast<BooleanImpl*>( impl )->value();
}

String Boolean::toString() const
{
    return impl->toString();
}

Boolean Boolean::toBoolean() const
{
    return *this;
}

//
// XPath Number type
//

Number::Number()
    : Value( new NumberImpl() )
{
}

Number::Number( ValueImpl *impl )
    : Value( impl )
{
}

Number::Number( const Number &src )
    : Value( src )
{
}

Number::Number( int value )
    : Value( new NumberImpl( value ) )
{
}

Number::Number( uint value )
    : Value( new NumberImpl( value ) )
{
}

Number::Number( double value )
    : Value( new NumberImpl( value ) )
{
}

Number::~Number()
{
}

Number &Number::operator=( const Number &other )
{
    Value::operator =(other);
    return *this;
}

double Number::value() const
{
    return static_cast<NumberImpl*>( impl )->value();
}

int Number::intValue() const
{
    return static_cast<NumberImpl*>( impl )->intValue();
}

String Number::toString() const
{
    return impl->toString();
}

Boolean Number::toBoolean() const
{
    return impl->toBoolean();
}

//
// XPath String type
//

String::String()
    : Value( new StringImpl() )
{
}

String::String( ValueImpl *impl )
    : Value( impl )
{
}

String::String( const char *value )
    : Value( new StringImpl( value ) )
{
}

String::String( const String &src )
    : Value( src )
{
}

String::String( const DOM::DOMString &s )
    : Value( new StringImpl( s ) )
{
}

String::~String()
{
}

String &String::operator=( const String &other )
{
    Value::operator =(other);
    return *this;
}

DOM::DOMString String::value() const
{
    return static_cast<StringImpl*>( impl )->value();
}

QString String::qstring() const
{
    return static_cast<StringImpl*>( impl )->qstring();
}

} // namespace XPath

// Local Variables:
// c-basic-offset: 4
// End:
