//  -*- c++ -*-

/*
 *  This file is part of the KDE libraries
 *  Copyright (C) 2004 Richard Moore, rich@kde.org
 *  Copyright (C) 2004 Zack Rusin, zack@kde.org
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef XPATH_VALUE_H
#define XPATH_VALUE_H

#include "xpath_valueimpl.h"

namespace XPath {

class NodeSet;
class Boolean;
class Number;
class String;

/**
 * The base-class of all value types used in the XPath data model.
 */
class Value
{
public:
    enum ValueType {
	ValueNodeSet, ValueBoolean, ValueNumber, ValueString
    };

    Value( const Value &res );
    virtual ~Value();
    Value &operator=( const Value &other );

    unsigned short type() const { return impl->type(); }
    bool isString() const { return  (impl->type() == ValueString); }
    bool isBoolean() const { return (impl->type() == ValueBoolean); }
    bool isNumber() const { return  (impl->type() == ValueNumber); }
    bool isNodeSet() const { return (impl->type() == ValueNodeSet); }

    virtual String toString() const;
    virtual Boolean toBoolean() const;
    virtual Number toNumber() const;

protected:
    Value( ValueImpl *impl );
protected:
    ValueImpl *impl;
};

/**
 * Represents an unordered collection of Nodes.
 * See XPath TR 3.3.
 */
class NodeSet : public Value
{
public:
    NodeSet();
    ~NodeSet();
    NodeSet &operator=( const NodeSet &other );

    DOM::NodeList value() const;
public:
    NodeSet( ValueImpl *impl );
};

/**
 * Represents a boolean value.
 * See XPath TR 3.4.
 */
class Boolean : public Value
{
public:
    Boolean();
    Boolean( ValueImpl * );
    Boolean( const Boolean &src );
    Boolean( bool value );
    ~Boolean();
    Boolean &operator=( const Boolean &other );

    bool value() const;

    /**
     * Converts the Boolean to a String. See the string() function in TR 4.2 for
     * details of the conventions used.
     */
    String toString() const;

    /** This is a no-op. */
    Boolean toBoolean() const;
};

/**
 * Represents a numeric value (floating point).
 * See XPath TR 3.5.
 */
class Number : public Value
{
public:
    Number();
    Number( ValueImpl* );
    Number( const Number &number );
    Number( int value );
    Number( uint value );
    Number( double value );

    ~Number();

    Number &operator=( const Number &other );

    double value() const;
    int intValue() const;

    /**
     * Converts the Number to a String. See the string() function in TR 4.2 for
     * details of the conventions used.
     */
    virtual String toString() const;

    /**
     * Converts the Number to a Boolean. See the boolean() function in TR 4.3
     * for details of the conventions used.
     */
    virtual Boolean toBoolean() const;
};

/**
 * Represents a string value.
 * See XPath TR 3.6.
 */
class String : public Value
{
public:
    String();
    String( ValueImpl* );
    String( const String &string );
    String( const char *value );
    String( const DOM::DOMString &value );
    ~String();

    String &operator=( const String &other );

    DOM::DOMString value() const;
    QString qstring() const;
};

} // namespace XPath

#endif // XPATH_VALUE_H

// Local Variables:
// c-basic-offset: 4
// End:
