/*
 *  This file is part of the KDE libraries
 *  Copyright (C) 2004 Richard Moore, rich@kde.org
 *  Copyright (C) 2004 Zack Rusin, zack@kde.org
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <float.h>
#include <math.h>
#include <qstring.h>

#include "xpath_valueimpl.h"
#include "xpath_value.h"

namespace XPath {

//
// XPath NodeSet type
//

NodeSetImpl::NodeSetImpl()
    : ValueImpl( Value::ValueNodeSet )
{
}

NodeSetImpl::~NodeSetImpl()
{
}

//
// XPath Boolean type
//

BooleanImpl::BooleanImpl()
    : ValueImpl( Value::ValueBoolean ),
      m_value( false )
{
}

BooleanImpl::BooleanImpl( bool value )
    : ValueImpl( Value::ValueBoolean ),
      m_value( value )
{
}

BooleanImpl::~BooleanImpl()
{
}

StringImpl *BooleanImpl::toString()
{
    if ( m_value )
	return new StringImpl( DOM::DOMString("true") );

    return new StringImpl( DOM::DOMString("false") );
}

BooleanImpl *BooleanImpl::toBoolean()
{
    return this;
}

//
// XPath Number type
//

NumberImpl::NumberImpl()
    : ValueImpl( Value::ValueNumber ),
      m_value( 0 )
{
}

NumberImpl::NumberImpl( uint value )
    : ValueImpl( Value::ValueNumber ),
      m_value( value )
{
}

NumberImpl::NumberImpl( int value )
    : ValueImpl( Value::ValueNumber ),
      m_value( value )
{
}

NumberImpl::NumberImpl( double value )
    : ValueImpl( Value::ValueNumber ),
      m_value( value )
{
}

NumberImpl::~NumberImpl()
{
}

int NumberImpl::intValue() const
{
    return (int) m_value;
}

StringImpl *NumberImpl::toString()
{
    DOM::DOMString s;

    if ( finite(m_value) ) {
	double intval = round( m_value );
	double diff = fabs( m_value-intval );

	if ( diff < DBL_EPSILON )
	    s = DOM::DOMString( QString::number( (int) intval ) );
	else
	    s = DOM::DOMString( QString::number( m_value ) );
    }
    else {
	// Handle non-numbers as specified by the string() function
	if ( isnan(m_value) )
	    s = DOM::DOMString("NaN");
	if ( isinf(m_value) == 1 )
	    s = DOM::DOMString("Infinity");
	if ( isinf(m_value) == -1 )
	    s = DOM::DOMString("-Infinity");
    }

    return new StringImpl( s );
}

BooleanImpl *NumberImpl::toBoolean()
{
    if ( isnan(m_value) )
	return new BooleanImpl( false );
    else if ( fabs(m_value) < DBL_EPSILON )
	return new BooleanImpl( false );
    else
	return new BooleanImpl( true );
}

//
// XPath String type
//

StringImpl::StringImpl()
    : ValueImpl( Value::ValueString )
{
}

StringImpl::StringImpl( const char *value )
    : ValueImpl( Value::ValueString ),
      m_value( DOM::DOMString( value ) )
{
}

StringImpl::StringImpl( const DOM::DOMString &s )
    : ValueImpl( Value::ValueString ),
      m_value(s)
{
}

StringImpl::~StringImpl()
{
}

QString StringImpl::qstring() const
{
    return m_value.string();
}

StringImpl *StringImpl::toString()
{
    return this;
}

} // namespace XPath

// Local Variables:
// c-basic-offset: 4
// End:
