/*
 *  This file is part of the KDE libraries
 *  Copyright (C) 2004 Richard Moore, rich@kde.org
 *  Copyright (C) 2004 Zack Rusin, zack@kde.org
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef XPATH_VALUEIMPL_H
#define XPATH_VALUEIMPL_H

#include "dom/dom_node.h"
#include "dom/dom_string.h"
#include "misc/shared.h"

namespace XPath {

class NodeSetImpl;
class BooleanImpl;
class NumberImpl;
class StringImpl;

class ValueImpl : public khtml::Shared<ValueImpl>
{
public:
    ValueImpl( unsigned short type ) : m_type( type ) { }
    virtual ~ValueImpl() {}

    unsigned short type() const { return m_type; }

    virtual StringImpl  *toString()  { return 0; }
    virtual BooleanImpl *toBoolean() { return 0; }
    virtual NumberImpl  *toNumber()  { return 0; }

private:
    unsigned short m_type;
};

class NodeSetImpl : public ValueImpl
{
public:
    NodeSetImpl();
    ~NodeSetImpl();
    DOM::NodeList value() const { return m_value; }
private:
    DOM::NodeList m_value;
};

class BooleanImpl : public ValueImpl
{
public:
    BooleanImpl();
    BooleanImpl( bool value );
    ~BooleanImpl();

    /**
     * Converts the Boolean to a String. See the string() function in TR 4.2 for
     * details of the conventions used.
     */
    StringImpl  *toString();

    /** This is a no-op. */
    BooleanImpl *toBoolean();

    bool value() const { return m_value; }
private:
    bool m_value;
};

/**
 * Represents a numeric value (floating point).
 * See XPath TR 3.5.
 */
class NumberImpl : public ValueImpl
{
public:
    NumberImpl();
    NumberImpl( int value );
    NumberImpl( uint value );
    NumberImpl( double value );

    ~NumberImpl();

    double value() const { return m_value; }
    int intValue() const;

    /**
     * Converts the Number to a String. See the string() function in TR 4.2 for
     * details of the conventions used.
     */
    virtual StringImpl *toString();

    /**
     * Converts the Number to a Boolean. See the boolean() function in TR 4.3
     * for details of the conventions used.
     */
    virtual BooleanImpl *toBoolean();
private:
    double m_value;
};

/**
 * Represents a string value.
 * See XPath TR 3.6.
 */
class StringImpl : public ValueImpl
{
public:
    StringImpl();
    StringImpl( const char *value );
    StringImpl( const DOM::DOMString &value );
    ~StringImpl();

    DOM::DOMString value() const { return m_value; }
    QString qstring() const;

    /** This is a no-op. */
    virtual StringImpl *toString();
private:
    DOM::DOMString m_value;
};

}

#endif

// Local Variables:
// c-basic-offset: 4
// End:
